/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2006)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: aversive_timers.h,v 1.2 2006-02-21 23:05:18 zer0 Exp $
 *
 */

// Microb Technology & EirBOT
// This file contains definitions used for timer use

/* to be completed with other archs 
 * XXX fix name of PWM, we don't want to redefine anything */


#ifndef _TIMERS_
#define _TIMERS_



// ATMEGA128 ////////////////////////////////////////
#if defined (__AVR_ATmega128__)

// available pwm
#define PWM0   0
#define PWM1A  1
#define PWM1B  2
#define PWM1C  3
#define PWM2   4
#define PWM3A  5
#define PWM3B  6
#define PWM3C  7


// OCR_BITS
#define OCR0_DDR DDRB
#define OCR0_BIT 4
#define OCR1A_DDR DDRB
#define OCR1A_BIT 5
#define OCR1B_DDR DDRB
#define OCR1B_BIT 6
#define OCR1C_DDR DDRB
#define OCR1C_BIT 7
#define OCR2_DDR DDRB
#define OCR2_BIT 7
#define OCR3A_DDR DDRE
#define OCR3A_BIT 3
#define OCR3B_DDR DDRE
#define OCR3B_BIT 4
#define OCR3C_DDR DDRE
#define OCR3C_BIT 5



// Timer WGM bits
#define TIMER_8_MODE_NORMAL      0
#define TIMER_8_MODE_PWM_PC      1 // phase correct PWM
#define TIMER_8_MODE_CTC         2
#define TIMER_8_MODE_PWM         3 // fast PWM

#define TIMER_16_MODE_NORMAL      0
#define TIMER_16_MODE_PWM_PC_8    1 // phase correct PWM 8 bits
#define TIMER_16_MODE_PWM_PC_9    2 // phase correct PWM 9 bits
#define TIMER_16_MODE_PWM_PC_10   3 // phase correct PWM 10 bits
#define TIMER_16_MODE_CTC1        4 // clear on compare, TOP = OCRA
#define TIMER_16_MODE_PWM_8       5 // fast PWM 8 bits
#define TIMER_16_MODE_PWM_9       6 // fast PWM 9 bits
#define TIMER_16_MODE_PWM_10      7 // fast PWM 10 bits
#define TIMER_16_MODE_PWM_PFC1    8 // PWM, Phase & Freq Correct
#define TIMER_16_MODE_PWM_PFC2    9 // PWM, Phase & Freq Correct
#define TIMER_16_MODE_PWM_PC1     10 // PWM, Phase Correct
#define TIMER_16_MODE_PWM_PC2     11 // PWM, Phase Correct
#define TIMER_16_MODE_CTC2        12 // clear on compare, TOP = ICR
#define TIMER_16_MODE_PWM_F1      14 // fast PWM
#define TIMER_16_MODE_PWM_F2      15 // fast PWM

// prescaler options
// timer 0
#define TIMER_8_PRESCALE_STOP 0
#define TIMER_8_PRESCALE_1    1
#define TIMER_8_PRESCALE_8    2
#define TIMER_8_PRESCALE_32   3
#define TIMER_8_PRESCALE_64   4
#define TIMER_8_PRESCALE_128  5
#define TIMER_8_PRESCALE_256  6
#define TIMER_8_PRESCALE_1024 7

// timer 2 is different !! FIXME

// timer 1
#define TIMER_16_PRESCALE_STOP 0
#define TIMER_16_PRESCALE_1    1
#define TIMER_16_PRESCALE_8    2
#define TIMER_16_PRESCALE_64   3
#define TIMER_16_PRESCALE_256  4
#define TIMER_16_PRESCALE_1024 5
#define TIMER_16_PRESCALE_EXT_FALL 6 // falling edge on T1
#define TIMER_16_PRESCALE_EXT_RISE 7 // rising edge on T1


// ATMEGA32 ////////////////////////////////////////
#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega323__)

#ifdef __AVR_ATmega323__
// renamed bits
//#define WGM20 PWM2 // not ok coz redefinition of PWM2
#define WGM20 6
#define WGM21 CTC2

#define WGM10 PWM10
#define WGM11 PWM11
#define WGM12 3//CTC1
#define WGM13 4 // reserve bit

//#define WGM00 PWM0 // not ok coz redefinition of PWM2
#define WGM00 6
#define WGM01 CTC0

// we need this label for ourselves
#ifdef PWM2
#undef PWM2
#endif
#ifdef PWM0
#undef PWM0
#endif

#endif // mega323


// available pwm
#define PWM0  0
#define PWM1A 1
#define PWM1B 2
#define PWM2  3


// OCR_BITS
#define OCR0_DDR DDRB
#define OCR0_BIT 3
#define OCR1A_DDR DDRD
#define OCR1A_BIT 5
#define OCR1B_DDR DDRD
#define OCR1B_BIT 4
#define OCR2_DDR DDRD
#define OCR2_BIT 7

// Timer WGM bits
#define TIMER_8_MODE_NORMAL      0
#define TIMER_8_MODE_PWM_PC      1 // phase correct PWM
#define TIMER_8_MODE_CTC         2
#define TIMER_8_MODE_PWM         3 // fast PWM

#define TIMER_16_MODE_NORMAL      0
#define TIMER_16_MODE_PWM_PC_8    1 // phase correct PWM 8 bits
#define TIMER_16_MODE_PWM_PC_9    2 // phase correct PWM 9 bits
#define TIMER_16_MODE_PWM_PC_10   3 // phase correct PWM 10 bits
#define TIMER_16_MODE_CTC1        4 // clear on compare, TOP = OCRA
#define TIMER_16_MODE_PWM_8       5 // fast PWM 8 bits
#define TIMER_16_MODE_PWM_9       6 // fast PWM 9 bits
#define TIMER_16_MODE_PWM_10      7 // fast PWM 10 bits
#define TIMER_16_MODE_PWM_PFC1    8 // PWM, Phase & Freq Correct
#define TIMER_16_MODE_PWM_PFC2    9 // PWM, Phase & Freq Correct
#define TIMER_16_MODE_PWM_PC1     10 // PWM, Phase Correct
#define TIMER_16_MODE_PWM_PC2     11 // PWM, Phase Correct
#define TIMER_16_MODE_CTC2        12 // clear on compare, TOP = ICR
#define TIMER_16_MODE_PWM_F1      14 // fast PWM
#define TIMER_16_MODE_PWM_F2      15 // fast PWM


// prescaler options
// timer 0
#define TIMER_8_PRESCALE_STOP 0
#define TIMER_8_PRESCALE_1    1
#define TIMER_8_PRESCALE_8    2
#define TIMER_8_PRESCALE_64   3
#define TIMER_8_PRESCALE_256  4
#define TIMER_8_PRESCALE_1024 5
#define TIMER_8_PRESCALE_EXT_FALL 6 // falling edge on T0
#define TIMER_8_PRESCALE_EXT_RISE 7 // rising edge on T0

// timer 1
#define TIMER_16_PRESCALE_STOP 0
#define TIMER_16_PRESCALE_1    1
#define TIMER_16_PRESCALE_8    2
#define TIMER_16_PRESCALE_64   3
#define TIMER_16_PRESCALE_256  4
#define TIMER_16_PRESCALE_1024 5
#define TIMER_16_PRESCALE_EXT_FALL 6 // falling edge on T1
#define TIMER_16_PRESCALE_EXT_RISE 7 // rising edge on T1

// TIMER 2 is different !!

// ATMEGA163 ////////////////////////////////////////
#elif defined (__AVR_ATmega163__)

// renamed bits
//#define WGM20 PWM2 // not ok coz redefinition of PWM2
#define WGM20 6
#define WGM21 CTC2

#define WGM10 PWM10
#define WGM11 PWM11
#define WGM12 CTC1
#define WGM13 4 // reserve bit

// we need this label for ourselves
#ifdef PWM2
#undef PWM2
#endif

// available pwm
#define PWM1A 1
#define PWM1B 2
#define PWM2  3


// OCR_BITS
#define OCR1A_DDR DDRD
#define OCR1A_BIT 5
#define OCR1B_DDR DDRD
#define OCR1B_BIT 4
#define OCR2_DDR DDRD
#define OCR2_BIT 7

// Timer WGM bits
#define TIMER_8_MODE_NORMAL      0
#define TIMER_8_MODE_PWM_PC      1 // phase correct PWM
#define TIMER_8_MODE_CTC         2
#define TIMER_8_MODE_PWM         3 // fast PWM

#define TIMER_16_MODE_NORMAL      0
#define TIMER_16_MODE_PWM_PC_8    1 // phase correct PWM 8 bits
#define TIMER_16_MODE_PWM_PC_9    2 // phase correct PWM 9 bits
#define TIMER_16_MODE_PWM_PC_10   3 // phase correct PWM 10 bits
#define TIMER_16_MODE_CTC1        4 // clear on compare, TOP = OCRA
#define TIMER_16_MODE_PWM_8       5 // fast PWM 8 bits
#define TIMER_16_MODE_PWM_9       6 // fast PWM 9 bits
#define TIMER_16_MODE_PWM_10      7 // fast PWM 10 bits


// prescaler options
// timer 0
#define TIMER_8_PRESCALE_STOP 0
#define TIMER_8_PRESCALE_1    1
#define TIMER_8_PRESCALE_8    2
#define TIMER_8_PRESCALE_64   3
#define TIMER_8_PRESCALE_256  4
#define TIMER_8_PRESCALE_1024 5
#define TIMER_8_PRESCALE_EXT_FALL 6 // falling edge on T0
#define TIMER_8_PRESCALE_EXT_RISE 7 // rising edge on T0

// timer 1
#define TIMER_16_PRESCALE_STOP 0
#define TIMER_16_PRESCALE_1    1
#define TIMER_16_PRESCALE_8    2
#define TIMER_16_PRESCALE_64   3
#define TIMER_16_PRESCALE_256  4
#define TIMER_16_PRESCALE_1024 5
#define TIMER_16_PRESCALE_EXT_FALL 6 // falling edge on T1
#define TIMER_16_PRESCALE_EXT_RISE 7 // rising edge on T1



// ATMEGAx8 ////////////////////////////////////////
#elif defined (__AVR_ATmega48__) || defined (__AVR_ATmega88__) || defined (__AVR_ATmega168__)

// available pwm
#define PWM0A  0
#define PWM0B  1
#define PWM1A  2
#define PWM1B  3
#define PWM2A  4
#define PWM2B  5

// OCR_BITS
#define OCR0A_DDR DDRD
#define OCR0A_BIT 6
#define OCR0B_DDR DDRD
#define OCR0B_BIT 5

#define OCR1A_DDR DDRB
#define OCR1A_BIT 1
#define OCR1B_DDR DDRB
#define OCR1B_BIT 2

#define OCR2A_DDR DDRB
#define OCR2A_BIT 3
#define OCR2B_DDR DDRD
#define OCR2B_BIT 3

// Timer WGM bits
#define TIMER_8_MODE_NORMAL      0
#define TIMER_8_MODE_PWM_PC      1 // phase correct PWM
#define TIMER_8_MODE_CTC         2
#define TIMER_8_MODE_PWM         3 // fast PWM

#define TIMER_16_MODE_NORMAL      0
#define TIMER_16_MODE_PWM_PC_8    1 // phase correct PWM 8 bits
#define TIMER_16_MODE_PWM_PC_9    2 // phase correct PWM 9 bits
#define TIMER_16_MODE_PWM_PC_10   3 // phase correct PWM 10 bits
#define TIMER_16_MODE_CTC1        4 // clear on compare, TOP = OCRA
#define TIMER_16_MODE_PWM_8       5 // fast PWM 8 bits
#define TIMER_16_MODE_PWM_9       6 // fast PWM 9 bits
#define TIMER_16_MODE_PWM_10      7 // fast PWM 10 bits



// prescaler options

// timer 0
#define TIMER_8_PRESCALE_STOP 0
#define TIMER_8_PRESCALE_1    1
#define TIMER_8_PRESCALE_8    2
#define TIMER_8_PRESCALE_32   3
#define TIMER_8_PRESCALE_64   4
#define TIMER_8_PRESCALE_128  5
#define TIMER_8_PRESCALE_256  6
#define TIMER_8_PRESCALE_1024 7

// timer 1
#define TIMER_16_PRESCALE_STOP 0
#define TIMER_16_PRESCALE_1    1
#define TIMER_16_PRESCALE_8    2
#define TIMER_16_PRESCALE_64   3
#define TIMER_16_PRESCALE_256  4
#define TIMER_16_PRESCALE_1024 5
#define TIMER_16_PRESCALE_EXT_FALL 6 // falling edge on T0
#define TIMER_16_PRESCALE_EXT_RISE 7 // rising edge on T0

#else
#error No timer/prescaler definitions for your AVR type
#endif

#endif
