/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: f16.c,v 1.2 2006-02-05 23:58:06 zer0 Exp $
 *
 */

#include <stdio.h>
#include <f16.h>

/* value of 2^^8 in float */
#define POW2_8F (256.0)


/**************** double <-> f16 */

f16 f16_from_double(double d)
{
    f16 f;
    f.integer = (d >= 0 ? (s08)d : (s08)d-1) ;
    f.decimal = (u08) ((d - (s08)d) * POW2_8F);
    
    return f;
}

double f16_to_double(f16 f)
{
    double d;
    
    d = f.integer;
    d += ((double)f.decimal / POW2_8F);
  
    return d;
}


/**************** s08,u08 <-> f16 */

f16 f16_from_integer(s08 i, u08 d)
{
    f16 f;
    f.integer = i;
    f.decimal = d;
    
    return f;
}

/**************** operations */

f16 f16_neg(f16 f)
{
    return s16_to_f16( -f16_to_s16(f) );
}

f16 f16_add(f16 a, f16 b)
{
    return s16_to_f16(  f16_to_s16(a) + f16_to_s16(b) );
}

f16 f16_sub(f16 a, f16 b)
{
    return s16_to_f16(  f16_to_s16(a) - f16_to_s16(b) );
}

f16 f16_inv(f16 f)
{
    if (F16_IS_ZERO(f))
        return F16_NAN;

    return s16_to_f16( ((s16)0x7fff) / (f16_to_s16(f)/2) );
}

f16 f16_mul(f16 a, f16 b)
{
    return s16_to_f16(   ( ((s16)(a.integer) * (s16)(b.integer)) << 8 )    +
                            (s16)(a.integer) * (s16)(b.decimal)             +
                            (s16)(a.decimal) * (s16)(b.integer)             +
                         ( ((s16)(a.decimal) * (s16)(b.decimal)) >> 8 ) );
}

f16 f16_div(f16 a, f16 b)
{
    if (F16_IS_ZERO(b))
        return F16_NAN;
    return f16_mul(a,f16_inv(b));
}

/* could be optimized... */
static inline u16  u16_sqrt(u16 x)
{
    u16 root=0x80;
    u16 mask=0x80;

    while(mask) {
        if( root*root < x ) {
            mask >>= 1;
            root |= mask;
        }
        else if ( root*root > x ) {
            root &= ~(mask);
            mask >>= 1;
            root |= mask;
            continue;
        }
        else 
            break;
    }
    return root;
}

f16 f16_sqrt(f16 f)
{
    u16 a,b,c,d;

    if (F16_IS_NEG(f))
        return F16_NAN;

    if(f.integer) {
        /* sqrt(a+b) = sqrt(a)*sqrt(1+b/a) */
        a=(u16)(f.integer) << 8 ;
        b=(u16)(f.decimal) << 8 ;
        c=u16_sqrt(a);
        d=u16_sqrt(0x100 + (b/a));
        return f16_mul(s16_to_f16( c<<4 ), s16_to_f16( d<<4 ));
    }
    else {
        b=(u16)(f.decimal) << 8 ;
        return s16_to_f16(u16_sqrt(b));
    }
}

/**************** debug */

void print_f16(f16 f)
{
    if(F16_IS_EQ(f, F16_NAN))
        printf("nan");
    else
        printf("%f",f16_to_double(f));
}

