/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: scheduler_common.c,v 1.8 2005-10-29 17:30:36 zer0 Exp $
 *
 */

#include <stdlib.h>
#include <utils.h>
#include <scheduler.h>

#include "scheduler_common.h"

/* not very very good, but acceptable for the moment */
#ifdef HOST_VERSION
#undef IRQ_LOCK
#undef IRQ_UNLOCK
#define IRQ_LOCK()
#define IRQ_UNLOCK()
#endif

u08 g_nb_events=0;

/** The global event structure */
struct event g_tab_event[SCHEDULER_NB_MAX_EVENT];

/** Initialisation of the module */
void scheduler_common_init(void)
{
  u08 i;

  for (i=0 ; i<SCHEDULER_NB_MAX_EVENT ; i++)
    {
      g_tab_event[i].period = 0;
      g_tab_event[i].current_time = 0;
      g_tab_event[i].priority = 0;
      g_tab_event[i].id = 0;
      g_tab_event[i].f = 0;
      g_tab_event[i].data = NULL;
    }
}

/** return a new id */
s08 scheduler_get_new_id(void)
{
  static s08 cur_id=0;
  u08 i;
  u08 j;

  for (i=0 ; i< 128 ; i++ ) {
    cur_id++;
    cur_id = (cur_id < 0 ? 0 : cur_id);
    for (j=0 ; j<SCHEDULER_NB_MAX_EVENT ; j++) {
      if(g_tab_event[j].id == cur_id)
	break;
    }
    if(j==SCHEDULER_NB_MAX_EVENT) {
      return cur_id;
    }
  }
  return -1;
}

/** return -1 on error or event number on success. */
s08 __scheduler_add_event(u08 unicity, void (*f)(void *), void * data, u16 period, u08 priority)
{
  u08 new_place;
  u08 i;
  
  IRQ_LOCK();

  /* too much events */
  if (g_nb_events == SCHEDULER_NB_MAX_EVENT)
    {
      IRQ_UNLOCK();
      return -1;
    }


  /* search the new place for the event in the table */
  for (i=0 ; i<g_nb_events ; i++)
    {
      if( g_tab_event[i].priority < priority )
	break;
    }
  new_place = i ;


  /* move the rest of the table */
  for (i=g_nb_events ; i>new_place ; i--)
    {
      g_tab_event[i].period = g_tab_event[i-1].period ;
      g_tab_event[i].current_time = g_tab_event[i-1].current_time;
      g_tab_event[i].priority = g_tab_event[i-1].priority;
      g_tab_event[i].id = g_tab_event[i-1].id;
      g_tab_event[i].f = g_tab_event[i-1].f;
      g_tab_event[i].data = g_tab_event[i-1].data;
    }

  /* adds the event */
  if(!unicity)
    g_tab_event[i].period = period ;
  else
    g_tab_event[i].period = 0 ;
    
  g_tab_event[i].current_time = period ;
  g_tab_event[i].priority = priority ;
  g_tab_event[i].id = scheduler_get_new_id();
  g_tab_event[i].f = f;
  g_tab_event[i].data = data;

  g_nb_events++;
  IRQ_UNLOCK();
  return g_tab_event[i].id;  
}

/* delete the event 'num'. return 0 on success */
s08 scheduler_del_event(s08 num)
{
  u08 i=0;

  IRQ_LOCK();

  /* search the event */
  while( i<g_nb_events && g_tab_event[i].id!=num ) 
    {
      i++ ;
    }

  /* not found */
  if(i==g_nb_events) {
    IRQ_UNLOCK();
    return -1;
  }

  /* move the rest of the table */
  for ( ; i<g_nb_events-1 ; i++)
    {
      g_tab_event[i].period = g_tab_event[i+1].period ;
      g_tab_event[i].current_time = g_tab_event[i+1].current_time;
      g_tab_event[i].priority = g_tab_event[i+1].priority;
      g_tab_event[i].id = g_tab_event[i+1].id;
      g_tab_event[i].f = g_tab_event[i+1].f;
      g_tab_event[i].data = g_tab_event[i+1].data;
    }
  g_tab_event[i].period = 0 ;
  g_tab_event[i].current_time = 0 ;
  g_tab_event[i].priority = 0 ;
  g_tab_event[i].id = 0 ;
  g_tab_event[i].f = 0 ;
  g_tab_event[i].data = NULL ;

  g_nb_events--;

  IRQ_UNLOCK();
  return 0;
}
