/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: scheduler_common.h,v 1.5 2005-10-29 17:30:36 zer0 Exp $
 *
 */

#define SCHEDULER_NB_STACKING_MAX SCHEDULER_NB_MAX_EVENT

/* not very very good, but acceptable for the moment */
#ifdef HOST_VERSION
#undef IRQ_LOCK
#undef IRQ_UNLOCK
#define IRQ_LOCK()
#define IRQ_UNLOCK()
#endif

/** The event structure */
struct event
{
	void (*f)(void *);  /**< a pointer to the schelduled function */
	void * data;      /**< a pointer to the data parameters */
	u16 period;       /**< interval between each call */
	u16 current_time; /**< time remaining before next call */
	u08 priority;     /**< if many events occur at the same time, the first to be executed will be the one with the highest priority */
	s08 id;           /**< identification of an event. For the moment, this module doesn't accept to create more than 255 events. */
};

extern u08 g_nb_events;

/** The global event structure */
extern struct event g_tab_event[SCHEDULER_NB_MAX_EVENT];

static inline void scheduler_common_interrupt(void)
{
	u08 i;
	static volatile u08 nb_stacking = 0;
	static volatile u08 running_priority = 0;
	u08 priority_temp;
  
	void (*event[SCHEDULER_NB_MAX_EVENT])(void *);
	void * event_data[SCHEDULER_NB_MAX_EVENT];
	u08 event_priority[SCHEDULER_NB_MAX_EVENT];
	u08 nb_events=0;

	/* too many interrupts, and events are too long :
	 *   -> don't exec events this time */
	IRQ_LOCK();
	if(nb_stacking > SCHEDULER_NB_STACKING_MAX) {
		IRQ_UNLOCK();
		/* FIXME : here we should log an error */
		return;
	}
	nb_stacking++;

	/* search active elements */
	for (i=0 ; i<SCHEDULER_NB_MAX_EVENT ; i++) {
	  
		/* timer is active ? */
		if(g_tab_event[i].current_time != 0) {
		  
			/* event must be executed, it is executed only if there is no
			 * stacking or if there is stacking but event priority is
			 * greater than current priority */
			if(g_tab_event[i].current_time == 1) {
				if ((nb_stacking > 1) && (g_tab_event[i].priority <= running_priority)) {
					continue;
				}
			}
		  
			g_tab_event[i].current_time --;
		  
			/* element timer has expired : we must exec the function */
			if(g_tab_event[i].current_time == 0) {
				event[nb_events] = g_tab_event[i].f ;
				event_data[nb_events] = g_tab_event[i].data ;
				event_priority[nb_events] = g_tab_event[i].priority ;
				nb_events ++;
			  
				/* reload event if necessary */
				if (g_tab_event[i].period) {
					g_tab_event[i].current_time = g_tab_event[i].period;
				}
				/* delete it if it is a single event */
				else {
					scheduler_del_event(g_tab_event[i].id);
					i--;
				}
			}
		}
	}
	IRQ_UNLOCK();

	/* execute events */
	for (i=0 ; i<nb_events ; i++) {
		priority_temp = running_priority ;
		IRQ_LOCK();
		running_priority = event_priority[i];
		IRQ_UNLOCK();
		event[i](event_data[i]);
		IRQ_LOCK();
		running_priority = priority_temp ;
		IRQ_UNLOCK();
	}

	IRQ_LOCK();
	nb_stacking --;
	IRQ_UNLOCK();
}
