/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: wait.h,v 1.11 2006-02-05 23:58:07 zer0 Exp $
 *
 */

/** \file modules/base/wait/wait.h
 *  \brief Interface of the wait module
 *
 * This file is an interface for wait functions, in order to put the 
 * microcontroller in a loop state.
 */

/**********************************************************/

#ifndef _WAIT_H_
#define _WAIT_H_

#include <utils.h>

#ifdef HOST_VERSION

#define wait_3cyc(n)
#define wait_4cyc(n)
#define wait_ms(n)

#else /* HOST_VERSION */


#include <avr/delay.h>

/** wait n "3 cycles time" 
 * n is 8 bits */
#define wait_3cyc(n) _delay_loop_1(n)

/** wait n "4 cycles time" 
 * n is 16 bits */
#define wait_4cyc(n) _delay_loop_2(n)

/** wait n milliseconds 
 * n is 16 bits */
extern void wait_ms(u16 n);


#endif /* else HOST_VERSION */

#endif /* _WAIT_ */
