/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: uart_archs.h,v 1.9 2005-11-11 22:44:52 larribe Exp $
 *
 */

/* Eirbot 2004 - Zer0
 * Register configuration for uart module
 */

/** \file uart_reg_defs.h
 *  \brief Register configuration for uart module
 *
 * This module provides :
 *   - Tx and Rx with fifo
 *   - two modes : the first one tries to be faster ; when you try to
 *     send data and the fifo is full, the byte is dropped. The second
 *     one (with UART1_DONT_LOOSE_DATA defined) can be slower ; when the 
 *     fifo is full, the writing of a data blocks in the interrupt.
 *   - Speed selection (for the moment the module don't use the UBRRxH
 *     register so the speed cannot be too low. (min is 4800 at 16 Mhz)
 *   - Parity selection (if the uC support it)
 *   - 5 to 9 data bits (if the uC support it). Warning : when you use 
 *     9 bits, the prototypes of the functions change (u08 become u16).
 *   - 1 or 2 stop bits (if the uC support it).
 *   - 2 UARTs (if the uC support it).
 * 
 *
 *
 * Number of bits in frame for tx and rx are the same
 *
 * It doesn't support some USART capabilities :
 *   - Synchronous mode
 *   - Multiprocessor communication
 */

#ifndef UART_ARCHS_H
#define UART_ARCHS_H


/******************************/

#if defined(__AVR_AT90S2313__) || defined(__AVR_AT90S8515__)
// uart style
#define UART_SIMPLE_UCR_USR 

/******************************/

#elif defined(__AVR_ATmega163__) 
// uart style
#define UART_SIMPLE_UCSR
#define UART_U2X

/******************************/

#elif defined (__AVR_ATmega128__) 
// uart style
#define UART_USART
#define UART_U2X
#define UART_DOUBLE

/******************************/

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega8515__) || defined (__AVR_ATmega323__) // verifier compat 323
// uart style
#define UART_USART
#define UART_U2X
#define UART_URSEL

/******************************/

#elif defined (__AVR_ATmega48__) || (__AVR_ATmega88__) || (__AVR_ATmega168__)
// uart style
#define UART_USART
#define UART_U2X

/******************************/

#elif defined (another_avr)
//add defs here


#else //avr type
#error This type of AVR is not supported by the UART Module. \
       Please Check the uart_archs.h file and in the uart.c\
       file ( in inits functions ) to resolve the problem.
#endif // avr type





#endif // UART_ARCHS_H
