/* Add Droids corp liscence here */


/* Droids-corp, Eirbot, Microb Techology 2005
 * Implementation for the control_system manager
 */

/** \file control_system_manager.h
 * \brief Interface for the control_system manager module.
 *
 * \todo Test the module on a real system.
 *
 * \test No test for the moment, only correct compilation.
 *
 * This module provide functions to control and regulate a system.
 */
    
/**/

#ifndef _CONTROL_SYSTEM_MANAGER_
#define _CONTROL_SYSTEM_MANAGER_

#include <stdlib.h> /* NULL */

#include <utils.h>  /* s32 */

/** The data structure used by the control_system_manager module */
struct cs {
        
        s32 (*consign_filter)(void *, s32);
        void* consign_filter_params;

        s32 (*correct_filter)(void*, s32);
        void* correct_filter_params;

        s32 (*feedback_filter)(void*, s32);
        void* feedback_filter_params;

        s32 (*process_out)(void*);
        void* process_out_params;

        void (*process_in)(void*, s32);
        void* process_in_params;

        s32 consign_value;
        s32 error_value;
        s32 out_value;
        
};

/******* - Prototyping - *******/

/** Initiate the control_system structure by setting all fields to NULL */
void cs_init(struct cs* cs);

/** Set the cs consign_filter fields in the cs structure */
void cs_set_consign_filter(struct cs* cs,
                                  s32 (*consign_filter)(void*, s32),
                                  void* consign_filter_params);

/** Set the cs correct_filter fields in the cs structure */
void  cs_set_correct_filter(struct cs* cs,
                                   s32 (*correct_filter)(void*, s32),
                                   void* correct_filer_params);

/** Set the cs feedback_filter fields in the cs structure */
void  cs_set_feedback_filter(struct cs* cs,
                                    s32 (*feedback_filter)(void*, s32),
                                    void* feedback_filer_params);

/** Set the cs process_in fields in the cs structure */
void cs_set_process_in(struct cs* cs,
                              void (*process_in)(void*, s32),
                              void* process_in_params);

/** Set the cs process_out fields in the cs structure */
void cs_set_process_out(struct cs* cs,
                               s32 (*process_out)(void*),
                               void* process_out_params);


/** \brief This function do the main loop of the control system process.
 *
 * - Save the consign in the structure.
 * - Apply the consign filter to the consign.
 * - Read the process out
 * - Apply the feedback filter to the process out
 * - Substract filtered consign to filtered process out.
 * - Save the result in error_value and apply the correct filter.
 * - Save the filtered result and send it to process_in().
 * - Return this result.
 * 
 */
s32 cs_do_process(struct cs* cs, s32 consign);

/** Apply cs_do_process() to the structure cs 
 * \param cs should be a (struct cs*)
 */
void cs_manage(void * cs);

/** Return the last process out send to process */
s32 cs_get_out( struct cs* cs);

/** Return the last calculated error */
s32 cs_get_error( struct cs* cs);

/** Return the current consign */
s32 cs_get_consign( struct cs* cs);

/** Change the consign without calculating control system */
void cs_set_consign( struct cs* cs, s32 v);


#endif /* #ifndef _CONTROL_SYSTEM_MANAGER_ */
