/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2006)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: pid.c,v 1.3 2006-02-05 23:58:07 zer0 Exp $
 *
 */

#include <pid.h>


/** this function will initialize all fieds of pid structure to 0 */
void pid_init (struct pid_filter * p)
{
        p->gain_P = 1 ;
        p->gain_I = 0 ;
        p->gain_D = 0 ; 

        p->out_shift = 0 ;

        /* 0 means desactivated */
        p->max_in = 0 ;
        p->max_I = 0 ;
        p->max_out = 0 ;

        p->integral = 0 ;
        p->last_in = 0 ;
}

void pid_set_gains(struct pid_filter *p, s16 gp, s16 gi, s16 gd)
{
    p->gain_P=gp;
    p->gain_I=gi;
    p->gain_D=gd;
}

void pid_set_maximums(struct pid_filter *p, s32 max_in, s32 max_I, s32 max_out)
{
    p->max_in=max_in;
    p->max_I=max_I;
    p->max_out=max_out;
}

void pid_set_out_shift(struct pid_filter *p, s16 out_shift)
{
    p->out_shift=out_shift;
}

s16 pid_get_gain_P(struct pid_filter *p)
{
    return (p->gain_P);
}

s16 pid_get_gain_I(struct pid_filter *p)
{
    return (p->gain_I);
}

s16 pid_get_gain_D(struct pid_filter *p)
{
    return (p->gain_D);
}


s16 pid_get_max_in(struct pid_filter *p)
{
    return (p->max_in);
}

s16 pid_get_max_I(struct pid_filter *p)
{
    return (p->max_I);
}

s16 pid_get_max_out(struct pid_filter *p)
{
    return (p->max_out);
}

s16 pid_get_out_shift(struct pid_filter *p)
{
    return (p->out_shift);
}

s32 pid_get_integral(struct pid_filter *p)
{
    return (p->integral);
}

s32 pid_get_last_in(struct pid_filter *p)
{
    return (p->last_in);
}

/* first parameter should be a (struct pid_filter *) */
s32 pid_do_filter(void * data, s32 in)
{
   s32 derivate ;
   s32 command ;
   struct pid_filter * p = data;
   
   /* Integral value */
   /* The integral become bigger with time .. (think to area of graph : we add one area to the previous) */
   /* so, integral = previous integral + current value */
   p->integral += in ;

   if (p->max_I)
       S_MAX(p->integral, p->max_I) ;

   /* derivate value                                                */
   /*             f(t+h) - f(t)        with f(t+h) = current value  */
   /*  derivate = -------------             f(t)   = previous value */
   /*                    h                                          */
   /* so derivate = current error - previous error                  */
   derivate = p->last_in - in ;

   if (p->max_in)
       S_MAX(derivate, p->max_in) ;

   /* so, command = P+coef_P
                    + I*coef_I
                    + D*coef_D */
   command =    in * p->gain_P 
              + p->integral * p->gain_I
              + derivate * p->gain_D ;

   command >>= p->out_shift ;

   if (p->max_out)
       S_MAX (command, p->max_out) ;


   /* backup of current error value (for the next calcul of derivate value) */
   p->last_in = in ;
   
   return command;
} 
