/* Eirbot & Microb Technology 2005 - 1/2 KL
 * Interface of the ADC Module
 */

/** \file adc.h
 *  \brief Interface of the ADC Module
 *
 *  \todo multi arch, filter config, frequency config, Differential Inputs
 *
 *  \test nothing done
 *
 * This module provides access to the built-in ADC unit of the ATMega uC.
 * With ATMega128, there is 8 analog input pins on port F.
 * A call to adc_init() is needed to activate the converter with the pins
 * specified in adc_config.h. This configuration file also sets the frequency
 * of the conversion (has to be set between 50 kHz and 200 kHz).
 * An interrupt handler stores each conversion in a "g_adc_results" vector,
 * whose size is the number of available analog inputs for the uC in use.
 * To access a result, call :
 *   u16 adc_get_result(u08 ADCi)
 * with ADCi=ADC0 ... ADCmax
 *
 * The value of each conversion is on the 10 least significant bits.
 *
 */


#ifndef _ADC_H_
#define _ADC_H_

#include <utils.h>
#include <adc_config.h>
#include <hardware/adc/adc_archs.h>

/**  Initialization of the module */
void adc_init(void);

#ifdef USE_ADC_SHUTDOWN
/**  Inibition of the module (for power reduction)*/
void adc_shutdown(void);
#endif // USE_ADC_SHUTDOWN

/** Get the value from ADCi  */
u16 adc_get_value(u08 ADCi);


#ifdef ADC_REF_EXT
#	define ADC_REF 0x00
	// 00xxxxxx
#elif defined ADC_REF_AVCC
#	define ADC_REF 0x40
	// 01xxxxxx
#elif defined ADC_REF_INTERNAL
#	define ADC_REF 0xC0
	// 11xxxxxx
#else 
#	error "you must specify an ADC reference in the adc_config.h"
#endif // ADC_REF_EXT

#endif // _ADC_H_
