/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2005)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: pwm.c,v 1.5 2006-02-21 23:05:19 zer0 Exp $
 *
 */

/* Droids-corp, Eirbot, Microb Technology 2005 - Zer0
 * Implementation for PWM
 */
/** \file pwm.c
 *  \brief Implementation for the PWM module.
 *
 *  \todo Test the module.
 *
 *  \test The modul fonctions and had been tested by Lamygalle.
 *
 * This module provides functions for using a pwm 
 */

#include <utils.h>

#include "pwm.h"

#include <aversive_timers.h>

#include <pwm_config.h>


#ifdef TIMER1_MODE

#if (TIMER1_MODE == TIMER_16_MODE_PWM_10)
#define TIMER1_PWM_BITS 10
#elif (TIMER1_MODE == TIMER_16_MODE_PWM_9)
#define TIMER1_PWM_BITS 9
#elif (TIMER1_MODE == TIMER_16_MODE_PWM_8)
#define TIMER1_PWM_BITS 8
#else
#error TIMER1 mode not valid, check pwm_config.h
#endif

#endif // TIMER1_MODE

#ifdef TIMER3_MODE

#if (TIMER3_MODE == TIMER_16_MODE_PWM_10)
#define TIMER3_PWM_BITS 10
#elif (TIMER3_MODE == TIMER_16_MODE_PWM_9)
#define TIMER3_PWM_BITS 9
#elif (TIMER3_MODE == TIMER_16_MODE_PWM_8)
#define TIMER3_PWM_BITS 8
#else
#error TIMER3 mode not valid, check pwm_config.h
#endif

#endif // TIMER3_MODE


/* FIXME : saturation des pwm ok ? */



#define pwm_timer_8bits_init(n, mode, prescale, timer_mode)              \
   TCCR##n  =  ( ( ( timer_mode & 0x01 )?( 1 << WGM##n##0 ):0   )  |     \
              ( ( timer_mode & 0x02 )?( 1 << WGM##n##1 ):0   )  |        \
              ( ( prescale  << CS##n##0  ) ) );                          \
  if(mode & PWM_REVERSE)                                                 \
    TCCR##n = ( ( (TCCR##n) & (~(0x03 << COM##n##0)) ) |                 \
		(0x01  << COM##n##0) );                                  \
  else                                                                   \
    TCCR##n = ( ( (TCCR##n) & (~(0x03 << COM##n##0)) ) |                 \
		(0x02  << COM##n##0) );                                  \
                                                                         \
                                                                         \
  OCR##n = 0x00;                                                         \
  sbi(OCR##n##_DDR,OCR##n##_BIT);                                        \
/* FIXME : correct if PWM_SIGN_BIT/PORT is not defined */                \
  if(mode & PWM_SIGNED)                                                  \
    {                                                                    \
      sbi(DDR(PWM##n##_SIGN_PORT), PWM##n##_SIGN_BIT);                   \
      cbi(PWM##n##_SIGN_PORT, PWM##n##_SIGN_BIT);                        \
    }                                                        

/***********************************************************/


#define pwm_timer_16bits_init(n,m, mode, prescale, timer_mode)           \
  TCCR##n##A =   (TCCR##n##A |                                           \
	        ( ( timer_mode & 0x01 )?( 1 << WGM##n##0 ):0   )  |      \
                ( ( timer_mode & 0x02 )?( 1 << WGM##n##1 ):0   )  )  ;   \
                                                                         \
  TCCR##n##B =  ( ( ( timer_mode & 0x04 )?( 1 << WGM##n##2 ):0   )  |    \
		  ( ( timer_mode & 0x08 )?( 1 << WGM##n##3 ):0   )  |    \
		  ( ( prescale  << CS##n##0  )  )  );                    \
                                                                         \
  if(mode & PWM_REVERSE)                                                 \
    TCCR##n##A = ( ( (TCCR##n##A) &                                      \
		             (~(0x03 << COM##n##m##0)) ) |               \
		             (0x01  << COM##n##m##0) );                  \
  else                                                                   \
    TCCR##n##A = ( ( (TCCR##n##A) &                                      \
		             (~(0x03 << COM##n##m##0)) ) |               \
		             (0x02  << COM##n##m##0) );                  \
                                                                         \
  OCR##n##m = 0;                                                         \
  sbi(OCR##n##m##_DDR,OCR##n##m##_BIT);                                  \
                                                                         \
  if(mode & PWM_SIGNED)                                                  \
    {                                                                    \
      sbi(DDR(PWM##n##m##_SIGN_PORT), PWM##n##m##_SIGN_BIT);             \
      cbi(PWM##n##m##_SIGN_PORT, PWM##n##m##_SIGN_BIT);                  \
    }                                                            

/***********************************************************/


#define pwm_timer_8bits_set(n, value, mode)                              \
  S_MAX(value, PWM_MAX);                                                 \
  if ( mode & PWM_SIGNED )                                               \
    {                                                                    \
    if (value < 0)                                                       \
      {                                                                  \
      if ( mode & PWM_SIGN_INVERTED )                                    \
        cbi(PWM##n##_SIGN_PORT, PWM##n##_SIGN_BIT);                      \
      else                                                               \
        sbi(PWM##n##_SIGN_PORT, PWM##n##_SIGN_BIT);                      \
	                                                                 \
        if ( mode & PWM_SPECIAL_SIGN_MODE )                              \
          value = value+((1<<8)-1);                                      \
        else                                                             \
          value = -value;                                                \
      }                                                                  \
    else                                                                 \
      {                                                                  \
      if ( mode & PWM_SIGN_INVERTED )                                    \
        sbi(PWM##n##_SIGN_PORT, PWM##n##_SIGN_BIT);                      \
      else                                                               \
        cbi(PWM##n##_SIGN_PORT, PWM##n##_SIGN_BIT);                      \
      }                                                                  \
    }                                                                    \
  else                                                                   \
    {                                                                    \
    MIN(value,0);                                                        \
    }                                                                    \
                                                                         \
  OCR##n =(u08) ( value >> ( PWM_SIGNIFICANT_BITS - 8 )) ;

/***********************************************************/
 

#define pwm_timer_16bits_set(n, m, value, mode)                          \
  S_MAX(value, PWM_MAX);                                                 \
  if ( mode & PWM_SIGNED )                                               \
    {                                                                    \
      if (value < 0)                                                     \
        {                                                                \
        if ( mode & PWM_SIGN_INVERTED )                                  \
          cbi(PWM##n##m##_SIGN_PORT, PWM##n##m##_SIGN_BIT);              \
        else                                                             \
          sbi(PWM##n##m##_SIGN_PORT, PWM##n##m##_SIGN_BIT);              \
                                                                         \
        if ( mode & PWM_SPECIAL_SIGN_MODE )                              \
          value = value+((1<<PWM_SIGNIFICANT_BITS)-1);                   \
        else                                                             \
          value = -value;                                                \
        }                                                                \
      else                                                               \
        {                                                                \
        if ( mode & PWM_SIGN_INVERTED )                                  \
          sbi(PWM##n##m##_SIGN_PORT, PWM##n##m##_SIGN_BIT);              \
        else                                                             \
          cbi(PWM##n##m##_SIGN_PORT, PWM##n##m##_SIGN_BIT);              \
        }                                                                \
    }                                                                    \
  else                                                                   \
    {                                                                    \
      MIN(value,0);                                                      \
    }                                                                    \
                                                                         \
  OCR##n##m = ( value >> ( PWM_SIGNIFICANT_BITS -  TIMER##n##_PWM_BITS )) ;                                                                    


/***********************************************************/

#if (defined PWM0) && (defined PWM0_ENABLED)
void pwm_init_0(void)
{
  IRQ_LOCK();
  pwm_timer_8bits_init(0,PWM0_MODE,TIMER0_PRESCALE, TIMER0_MODE);
  IRQ_UNLOCK();
}

/***********************************************************/

void pwm_set_0(s16 value)
{
  IRQ_LOCK();
  pwm_timer_8bits_set(0,value,PWM0_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM0) && (defined PWM0_ENABLED)

/***********************************************************/


/***********************************************************/

#if (defined PWM1A) && (defined PWM1A_ENABLED)
void pwm_init_1A(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(1,A,PWM1A_MODE,TIMER1_PRESCALE, TIMER1_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_1A(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(1,A,value,PWM1A_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM1A) && (defined PWM1A_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM1B) && (defined PWM1B_ENABLED)
void pwm_init_1B(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(1,B,PWM1B_MODE,TIMER1_PRESCALE, TIMER1_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_1B(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(1,B,value,PWM1B_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM1B) && (defined PWM1B_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM1C) && (defined PWM1C_ENABLED)
void pwm_init_1C(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(1,C,PWM1C_MODE,TIMER1_PRESCALE, TIMER1_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_1C(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(1,C,value, PWM1C_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM1C) && (defined PWM1C_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM2) && (defined PWM2_ENABLED)
void pwm_init_2(void)
{
  IRQ_LOCK();
  pwm_timer_8bits_init(2,PWM2_MODE,TIMER2_PRESCALE, TIMER2_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_2(s16 value)
{
  IRQ_LOCK();
  pwm_timer_8bits_set(2,value,PWM2_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM2) && (defined PWM2_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM3A) && (defined PWM3A_ENABLED)
void pwm_init_3A(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(3,A,PWM3A_MODE,TIMER3_PRESCALE, TIMER3_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_3A(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(3,A,value,PWM3A_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM3A) && (defined PWM3A_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM3B) && (defined PWM3B_ENABLED)
void pwm_init_3B(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(3,B,PWM3B_MODE,TIMER3_PRESCALE, TIMER3_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_3B(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(3,B,value,PWM3B_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM3B) && (defined PWM3B_ENABLED)



/***********************************************************/


/***********************************************************/

#if (defined PWM3C) && (defined PWM3C_ENABLED)
void pwm_init_3C(void)
{
  IRQ_LOCK();
  pwm_timer_16bits_init(3,C,PWM3C_MODE,TIMER3_PRESCALE, TIMER3_MODE);
  IRQ_UNLOCK();
}


/***********************************************************/

void pwm_set_3C(s16 value)
{
  IRQ_LOCK();
  pwm_timer_16bits_set(3,C,value,PWM3C_MODE);
  IRQ_UNLOCK();
}
#endif // (defined PWM3C) && (defined PWM3C_ENABLED)


/***********************************************************/


/***********************************************************/

// global init
void pwm_init(void)
{
#if (defined PWM0) && (defined PWM0_ENABLED)
  pwm_init_0();
#endif

#if (defined PWM1A) && (defined PWM1A_ENABLED)
  pwm_init_1A();
#endif
#if (defined PWM1B) && (defined PWM1B_ENABLED)
  pwm_init_1B();
#endif
#if (defined PWM1C) && (defined PWM1C_ENABLED)
  pwm_init_1C();
#endif

#if (defined PWM2) && (defined PWM2_ENABLED)
  pwm_init_2();
#endif

#if (defined PWM3A) && (defined PWM3A_ENABLED)
  pwm_init_3A();
#endif
#if (defined PWM3B) && (defined PWM3B_ENABLED)
  pwm_init_3B();
#endif
#if (defined PWM3C) && (defined PWM3C_ENABLED)
  pwm_init_3C();
#endif


#ifdef PWM_SYNCH
  /** this tries to make the PWMs to synchronize.
  experimental feature, could lead to problems.
  to synch PWMs you need to enshure that the timers have same prescales,
  and the same PWM mode  */
  
// calculation of the step
#ifdef PWM_SYNCH_NO_PRESCALE
  #define STEP 1
  #define NOP_STEP() nop()
#else
  #define STEP 0
  #define NOP_STEP() {}
#endif
  
  
  IRQ_LOCK();
  
  {
  // register declarations
#ifdef TIMER_0_SYNCH
  register u08 tccr0;
#endif
#ifdef TIMER_1_SYNCH
  register u08 tccr1b;
#endif
#ifdef TIMER_2_SYNCH
  register u08 tccr2;
#endif
#ifdef TIMER_3_SYNCH
  register u08 tccr3b;
#endif

  // save state and stop timers
#ifdef TIMER_0_SYNCH
  tccr0 = TCCR0;
  TCCR0 = 0x00;
#endif
#ifdef TIMER_1_SYNCH
 tccr1b = TCCR1B;
 TCCR1B = 0x00;
#endif
#ifdef TIMER_2_SYNCH
  tccr2 = TCCR2;
  TCCR2 = 0x00;
#endif
#ifdef TIMER_3_SYNCH
  tccr3b = TCCR3B;
  TCCR3B = 0x00;
#endif

  // the timers are now stopped, we set the TCNT's to given values
  // set timers to given values
  // the timers would then be startes one after another, and synchronize themselves

#ifdef TIMER_0_SYNCH
  TCNT0 = 0;
#endif
#ifdef TIMER_1_SYNCH
  TCNT1H = 0;
  TCNT1L = 1l * STEP;
#endif
#ifdef TIMER_2_SYNCH
  TCNT2 = 2l * STEP;
#endif
#ifdef TIMER_3_SYNCH
  TCNT3H = 0;
  TCNT3L = 3l * STEP;
#endif

  // resetting prescalers (all prescalers, enshure that there are no problems with that )
#ifdef PSR321 // mega 128
  sbi(SFIOR, PSR321);
#endif
#ifdef PSR0 // mega 128
  sbi(SFIOR, PSR0);
#endif
#ifdef PSR10 // mega  163, 32 ...
  sbi(SFIOR, PSR10);
#endif
#ifdef PSR2 // mega  163, 32 ...
  sbi(SFIOR, PSR2);
#endif

  // running timers
#ifdef TIMER_0_SYNCH
  TCCR0 = tccr0;
#else
  NOP_STEP();
#endif
#ifdef TIMER_1_SYNCH
  TCCR1B = tccr1b;
#else
  NOP_STEP();
#endif
#ifdef TIMER_2_SYNCH
  TCCR2 = tccr2;
#else
  NOP_STEP();
#endif
#ifdef TIMER_3_SYNCH
  TCCR3B = tccr3b;
#else
  NOP_STEP();
#endif

  }
  IRQ_UNLOCK();

#endif // PWM_SYNCH


}
/***********************************************************/


/***********************************************************/

// global SET FUNCTION
void pwm_set(void * data, s32 value)
{
  u08 num = (u08)(int)data;
  S_MAX(value, 0x7FFF);

  switch(num)
    {
#if (defined PWM0) && (defined PWM0_ENABLED)
    case PWM0:
      pwm_set_0(value);
      break;
#endif

#if (defined PWM1A) && (defined PWM1A_ENABLED)
    case PWM1A:
      pwm_set_1A(value);
      break;
#endif
#if (defined PWM1B) && (defined PWM1B_ENABLED)
    case PWM1B:
      pwm_set_1B(value);
      break;
#endif
#if (defined PWM1C) && (defined PWM1C_ENABLED)
    case PWM1C:
      pwm_set_1C(value);
      break;
#endif

#if (defined PWM2) && (defined PWM2_ENABLED)
    case PWM2:
      pwm_set_2(value);
      break;
#endif

#if (defined PWM3A) && (defined PWM3A_ENABLED)
    case PWM3A:
      pwm_set_3A(value);
      break;
#endif
#if (defined PWM3B) && (defined PWM3B_ENABLED)
    case PWM3B:
      pwm_set_3B(value);
      break;
#endif
#if (defined PWM3C) && (defined PWM3C_ENABLED)
    case PWM3C:
      pwm_set_3C(value);
      break;
#endif

    default:
      break;
    }

}



